@@ -1,3 +1,6 @@
 from beecolpy.beecolpy import abc
+from beecolpy.beecolpy import bin_abc
+
+# Deprecation warning
 from beecolpy.beecolpy import binabc
 from beecolpy.beecolpy import amabc
